@@ -95,12 +95,15 @@ int sr_session_switch_ds(sr_session_ctx_t *, sr_datastore_t);
 sr_datastore_t sr_session_get_ds(sr_session_ctx_t *);
 sr_conn_ctx_t *sr_session_get_connection(sr_session_ctx_t *);
 int sr_session_get_error(sr_session_ctx_t *, const sr_error_info_t **);
-int sr_session_set_error_message(sr_session_ctx_t *, const char *, ...);
+int sr_session_set_error(sr_session_ctx_t *, const char *, sr_error_t, const char *, ...);
 const char *sr_session_get_orig_name(sr_session_ctx_t *session);
 int sr_session_set_orig_name(sr_session_ctx_t *session, const char *);
 int sr_session_get_orig_data(sr_session_ctx_t *session, uint32_t idx, uint32_t *size, const void **data);
 int sr_session_push_orig_data(sr_session_ctx_t * session, uint32_t size, const void *data);
 
+int sr_lock(sr_session_ctx_t *session, const char *module_name, uint32_t timeout_ms);
+int sr_unlock(sr_session_ctx_t *session, const char *module_name);
+
 typedef enum sr_val_type_e {
 	SR_UNKNOWN_T,
 	SR_LIST_T,
@@ -188,9 +191,11 @@ void sr_release_data(sr_data_t *);
 int sr_rpc_send_tree(sr_session_ctx_t *, struct lyd_node *, uint32_t, sr_data_t **);
 
 int sr_set_item_str(sr_session_ctx_t *, const char *, const char *, const char *, const sr_edit_options_t);
+int sr_discard_items(sr_session_ctx_t *, const char *);
 int sr_delete_item(sr_session_ctx_t *, const char *, const sr_edit_options_t);
 int sr_oper_delete_item_str(sr_session_ctx_t *, const char *, const char *, const sr_edit_options_t);
 int sr_edit_batch(sr_session_ctx_t *, const struct lyd_node *, const char *);
+int sr_copy_config(sr_session_ctx_t *, const char *, sr_datastore_t, uint32_t);
 int sr_replace_config(sr_session_ctx_t *, const char *, struct lyd_node *, uint32_t);
 int sr_validate(sr_session_ctx_t *, const char *, uint32_t);
 int sr_apply_changes(sr_session_ctx_t *, uint32_t);
@@ -203,6 +208,9 @@ typedef enum sr_subscr_flag_e {
 	SR_SUBSCR_ENABLED,
 	SR_SUBSCR_UPDATE,
 	SR_SUBSCR_OPER_MERGE,
+	SR_SUBSCR_THREAD_SUSPEND,
+	SR_SUBSCR_OPER_POLL_DIFF,
+	SR_SUBSCR_FILTER_ORIG,
 	...
 } sr_subscr_flag_t;
 
@@ -284,3 +292,6 @@ int sr_notif_subscribe_tree(sr_session_ctx_t *, const char *module_name, const c
 	void *priv, sr_subscr_options_t, sr_subscription_ctx_t **);
 
 int sr_notif_send_tree(sr_session_ctx_t *, struct lyd_node *notif, uint32_t timeout_ms, int wait);
+
+typedef int... mode_t;
+int sr_get_module_ds_access(sr_conn_ctx_t *conn, const char *module_name, int mod_ds, char **owner, char **group, mode_t *perm);
