@@ -11,6 +11,7 @@ import libyang
 from _sysrepo import ffi, lib
 from .change import Change
 from .errors import (
+    SysrepoError,
     SysrepoInternalError,
     SysrepoNotFoundError,
     SysrepoUnsupportedError,
@@ -111,6 +112,42 @@ class SysrepoSession:
         """
         return datastore_name(lib.sr_session_get_ds(self.cdata))
 
+    def lock(self, module_name: str = "", timeout_ms: int = 0) -> None:
+        """
+        Locks the data of the specified module or the whole datastore.
+        :arg module_name:
+            Optional name of the module to be locked.
+        :arg timeout_ms:
+            Optional timeout in ms for waiting. If 0, no waiting is performed.
+        """
+        module = str2c(module_name) if len(module_name) > 0 else ffi.NULL
+        check_call(lib.sr_lock, self.cdata, module, timeout_ms)
+
+    def unlock(self, module_name: str = "") -> None:
+        """
+        Unlocks the data of the specified module or the whole datastore.
+        :arg module_name:
+            Optional name of the module to be locked.
+        """
+        module = str2c(module_name) if len(module_name) > 0 else ffi.NULL
+        check_call(lib.sr_unlock, self.cdata, module)
+
+    @contextmanager
+    def locked(self, module_name: str = "", timeout_ms: int = 0):
+        """
+        Convenience method which manages un-/locking the data of the specified module
+        or the whole datastore.
+        :arg module_name:
+            Optional name of the module to be locked.
+        :arg timeout_ms:
+            Optional timeout in ms for waiting. If 0, no waiting is performed.
+        """
+        try:
+            self.lock(module_name, timeout_ms)
+            yield
+        finally:
+            self.unlock(module_name)
+
     def switch_datastore(self, datastore: str) -> None:
         """
         Change datastore which the session operates on. All subsequent calls will be
@@ -128,7 +165,7 @@ class SysrepoSession:
         ds = datastore_value(datastore)
         check_call(lib.sr_session_switch_ds, self.cdata, ds)
 
-    def set_error(self, message: str):
+    def set_error(self, err: BaseException):
         """
         Set detailed error information into provided session. Used to notify the client
         library about errors that occurred in the application code. Does not print the
@@ -137,13 +174,24 @@ class SysrepoSession:
         Intended for change, RPC/action, or operational callbacks to be used on the
         provided session.
 
-        :arg str message:
-            The detailed error message.
+        :arg BaseException err:
+            The python exception object that carries the error information.
         """
         if not self.is_implicit:
             raise SysrepoUnsupportedError("can only report errors on implicit sessions")
+        if isinstance(err, SysrepoError):
+            code = err.rc
+            message = err.msg
+        else:
+            code = lib.SR_ERR_CALLBACK_FAILED
+            message = str(err)
         check_call(
-            lib.sr_session_set_error_message, self.cdata, str2c("%s"), str2c(message)
+            lib.sr_session_set_error,
+            self.cdata,
+            ffi.NULL,
+            code,
+            str2c("%s"),
+            str2c(message),
         )
 
     def get_originator_name(self) -> str:
@@ -279,6 +327,7 @@ class SysrepoSession:
     to reject changes.
     """
 
+    # pylint: disable=too-many-arguments
     def subscribe_module_change(
         self,
         module: str,
@@ -290,6 +339,8 @@ class SysrepoSession:
         passive: bool = False,
         done_only: bool = False,
         enabled: bool = False,
+        update: bool = False,
+        filter_origin: bool = False,
         private_data: Any = None,
         asyncio_register: bool = False,
         include_implicit_defaults: bool = True,
@@ -323,6 +374,11 @@ class SysrepoSession:
         :arg enabled:
             The subscriber wants to be notified about the current configuration
             at the moment of subscribing.
+        :arg update:
+            The subscriber wants to be called before the configuration is applied.
+        :arg filter_origin:
+            Filter events on the originator side to unburden the subscriber, but
+            results in 0 value for filtered-out changes in the subscriber infos.
         :arg private_data:
             Private context passed to the callback function, opaque to sysrepo.
         :arg asyncio_register:
@@ -354,7 +410,12 @@ class SysrepoSession:
         if asyncio_register:
             no_thread = True  # we manage our own event loop
         flags = _subscribe_flags(
-            no_thread=no_thread, passive=passive, done_only=done_only, enabled=enabled
+            no_thread=no_thread,
+            passive=passive,
+            done_only=done_only,
+            enabled=enabled,
+            update=update,
+            filter_origin=filter_origin,
         )
 
         check_call(
@@ -408,6 +469,8 @@ class SysrepoSession:
         passive: bool = False,
         done_only: bool = False,
         enabled: bool = False,
+        update: bool = False,
+        filter_origin: bool = False,
         private_data: Any = None,
         asyncio_register: bool = False,
     ) -> None:
@@ -442,6 +505,11 @@ class SysrepoSession:
         :arg enabled:
             The subscriber wants to be notified about the current configuration
             at the moment of subscribing.
+        :arg update:
+            The subscriber wants to be called before the configuration is applied.
+        :arg filter_origin:
+            Filter events on the originator side to unburden the subscriber, but
+            results in 0 value for filtered-out changes in the subscriber infos.
         :arg private_data:
             Private context passed to the callback function, opaque to sysrepo.
         :arg asyncio_register:
@@ -467,7 +535,12 @@ class SysrepoSession:
         if asyncio_register:
             no_thread = True  # we manage our own event loop
         flags = _subscribe_flags(
-            no_thread=no_thread, passive=passive, done_only=done_only, enabled=enabled
+            no_thread=no_thread,
+            passive=passive,
+            done_only=done_only,
+            enabled=enabled,
+            update=update,
+            filter_origin=filter_origin,
         )
         check_call(
             lib.sr_module_change_subscribe,
@@ -1159,6 +1232,18 @@ class SysrepoSession:
             lib.sr_set_item_str, self.cdata, str2c(xpath), str2c(value), ffi.NULL, 0
         )
 
+    def discard_items(self, xpath: str) -> None:
+        """
+        Prepare to discard nodes matching the specified xpath (or all if not
+        set) previously set by the session connection. Usable only for the
+        operational datastore . These changes are applied only after calling
+        apply_changes().
+
+        :arg xpath:
+            Path identifier of the data element to be deleted.
+        """
+        check_call(lib.sr_discard_items, self.cdata, str2c(xpath))
+
     def delete_item(self, xpath: str) -> None:
         """
         Prepare to delete the nodes matching the specified xpath. These changes
@@ -1302,6 +1387,35 @@ class SysrepoSession:
         dnode = module.parse_data_dict(config, strict=strict, validate=False)
         self.replace_config_ly(dnode, module_name, timeout_ms=timeout_ms)
 
+    def copy_config(
+        self, src_datastore: str, module_name: str = None, timeout_ms: int = 0
+    ) -> None:
+        """
+        Replaces a conventional datastore with the contents of another conventional
+        datastore. If the module is specified, limits the operation only to the
+        specified module. If it is not specified, the operation is performed on all
+        modules.
+
+        Note that copying from candidate to running or vice versa causes the candidate
+        datastore to revert to original behavior of mirroring running datastore
+        (datastores).
+
+        Required WRITE access.
+
+        :arg src_datastore:
+            Source datastore to copy configuration from. Can be one of `running`,
+            `startup`, `operational` or `candidate`. Must be a different datastore
+            than the session's.
+        :arg module_name:
+            Optional module name that limits the copy operation only to this module.
+        :arg timeout_ms:
+            Configuration callback timeout in milliseconds. If 0, default is used.
+        """
+        if self.is_implicit:
+            raise SysrepoUnsupportedError("cannot copy config from implicit sessions")
+        ds = datastore_value(src_datastore)
+        check_call(lib.sr_copy_config, self.cdata, str2c(module_name), ds, timeout_ms)
+
     def validate(self, module_name: str = None) -> None:
         """
         Perform the validation a datastore and any changes made in the current
@@ -1379,7 +1493,17 @@ class SysrepoSession:
         if not sr_data_p[0]:
             raise SysrepoInternalError("sr_rpc_send_tree returned NULL")
 
-        return self.new_dnode(sr_data_p[0].tree)
+        ctx = self.acquire_context()
+        dnode = libyang.DNode.new(ctx, sr_data_p[0].tree)
+
+        # customize the free method to use the sysrepo free
+        def sysrepo_free(dnode_src):
+            lib.sr_release_data(sr_data_p[0])
+            self.release_context()
+
+        dnode.free_func = sysrepo_free
+
+        return dnode
 
     def rpc_send(
         self,
@@ -1408,7 +1532,7 @@ class SysrepoSession:
         :arg strip_prefixes:
             If True, remove YANG module prefixes from dictionary keys.
         :arg include_implicit_defaults:
-            Include leaves with implicit default values in the retured dict.
+            Include leaves with implicit default values in the returned dict.
         :arg trim_default_values:
             Exclude leaves when their value equals the default.
         :arg keep_empty_containers:
@@ -1538,7 +1662,13 @@ def _get_oper_flags(no_state=False, no_config=False, no_subs=False, no_stored=Fa
 
 # -------------------------------------------------------------------------------------
 def _subscribe_flags(
-    no_thread=False, passive=False, done_only=False, enabled=False, oper_merge=False
+    no_thread=False,
+    passive=False,
+    done_only=False,
+    enabled=False,
+    oper_merge=False,
+    filter_origin=False,
+    update=False,
 ):
     flags = 0
     if no_thread:
@@ -1549,8 +1679,12 @@ def _subscribe_flags(
         flags |= lib.SR_SUBSCR_DONE_ONLY
     if enabled:
         flags |= lib.SR_SUBSCR_ENABLED
+    if update:
+        flags |= lib.SR_SUBSCR_UPDATE
     if oper_merge:
         flags |= lib.SR_SUBSCR_OPER_MERGE
+    if filter_origin:
+        flags |= lib.SR_SUBSCR_FILTER_ORIG
     return flags
 
 
